from .managers import (
    AvailabilityManager,
    MasterdataManager,
    MeteringManager,
    NWPManager,
    ResourceManager,
)
from .modeling import Coordinate, Group, Neighborhood, NWPParameter, NWPProvider, Plant



from dataclasses import dataclass
from enum import Enum


class NWPParameter(Enum):
    SOLAR = "solar"
    WIND = "wind"
    TEMPERATURE = "temperature"


class NWPProvider(Enum):
    ECMWF = "ecmwf"
    CONWX = "conwx"
    DMI = "dmi"


@dataclass(frozen=True)
class Coordinate:
    latitude: float
    longitude: float
    altitude: float


@dataclass(frozen=True)
class Neighborhood:
    coordinate: Coordinate
    num_neighbors: int



import numpy as np
import pandas as pd
import pvlib
from numpy.typing import NDArray

from domainiac.functions import conversions, interpolation
from domainiac.modeling import Coordinate


def clearsky_irradiance(
    times: NDArray[np.datetime64], coordinate: Coordinate
) -> NDArray[np.float64]:
    """Get the clearsky irradiance (W/m2) for a specific coordinate

    Args:
        times (NDArray[np.datetime64]): Times
        coordinate (Coordinate): Coordinate
    """

    location = pvlib.location.Location(coordinate.latitude, coordinate.longitude)

    # ensure 1D-array both when input is
    # scalar and 1D-array already
    array = conversions.as_array([times])
    if array.size != 1:
        array = array.squeeze()

    times = pd.DatetimeIndex(array)
    # pvlibs `get_clearsky` will return 0
    # if datetime unit is anything else than
    # nanoseconds (probably due to some
    # internal conversion going wrong)
    times = times.as_unit("ns")

    clearsky = location.get_clearsky(times)

    return clearsky["ghi"].to_numpy()


def interpolate_irradiance(
    coordinate: Coordinate, times: pd.Series, radiation_avg: pd.Series
):
    """Interpolate solar irradiance (i.e. instantaneous values) using
    binned index interpolation (using the clearsky irradiance as the reference profile)
    between observed values of solar radiance (i.e. binned average values)

    Args:
        coordinate (Coordinate): Coordinate
        times (pd.Series): Sequence of times defining the intervals of observations
        radiation (pd.Series): Observed values of radiation
    """

    def profile(x: NDArray[np.float64]) -> NDArray[np.float64]:
        times = conversions.float_to_datetime(x)
        return clearsky_irradiance(times, coordinate)

    x = conversions.datetime_to_float(times)
    y_avg = conversions.as_array(radiation_avg)
    # 5-minute step size should be sufficient for evaluating the clearsky integral
    quad_step_size = conversions.timedelta_to_float(pd.Timedelta("PT5M"))

    f = interpolation.binned_index_interpolation(profile, x, y_avg, quad_step_size)

    def estimate(times: pd.Series) -> NDArray[np.float64]:
        x = conversions.datetime_to_float(times)
        return f(x)

    return estimate



import numpy as np

ORIGIN = np.datetime64("2020-01-01")
UNIT = np.timedelta64(1, "s")


def as_array(values):
    return np.array(values)


def timedelta_to_float(values):
    return as_array(values).astype("timedelta64") / UNIT


def float_to_timedelta(values):
    return as_array(values) * UNIT


def datetime_to_float(values):
    return timedelta_to_float(as_array(values).astype("datetime64") - ORIGIN)


def float_to_datetime(values):
    return float_to_timedelta(values) + ORIGIN



from typing import Callable

import numpy as np
from numpy.typing import NDArray

IntegrableRealFunction = Callable[[NDArray[np.float64]], NDArray[np.float64]]

RealFunction = Callable[[NDArray[np.float64]], NDArray[np.float64]]



from .availability_manager import AvailabilityManager
from .masterdata_manager import MasterdataManager
from .metering_manager import MeteringManager
from .nwp_manager import NWPManager
from .outage_manager import OutageManager
from .resource_manager import ResourceManager
